<?php
/*

(w) 2025 Joachim Nickel – chop solutions GmbH
Lizenziert unter GNU GPL3 https://www.gnu.org/licenses/gpl-3.0.html

Bereitgestellt im Rahmen der Dokumentation für die Einrichtung
eines Cronjobs zur Archivierung von Matomo.

Siehe: https://www.chop.de/matomo/cronjob-archivierung/

Das folgende PHP-Skript wird versuchen, alle notwendigen Informationen
für die Einrichtung eines Cronjobs zum PHP-Script "console" zu ermitteln.
Installiere dieses Script bitte im Verzeichnis von Matomo. Nur wenn das
Script den Konsolenbefehl findet und prüfen kann, welche Matomo-Version
installiert ist, wird es dir den gesamten Befehl incl. Angabe des 
PHP-Interpreters ausgeben.

Hinweis: Dieses Skript wird ohne Gewähr bereitgestellt.

Das Script wird nur lokal ausgeführt und sammelt keine Daten!
Der Urheber Joachim Nickel wird nicht darüber informiert, dass du
dieses Script verwendest und wo du es verwendest.
Es bleibt also alles anonym. Versprochen.

ABER: Joachim Nickel würde sich sehr über deine Rückmeldung freuen, wenn
dieses Script dir bei der Einrichtung des Cronjobs behilflich war.
Schreib ihm gerne eine E-Mail an info@chop.de

Auch wenn es sehr unwahrscheinlich ist, dass folgende Passage relevant ist:
Mit der Nutzung erklärst du dich einverstanden, dass eine mögliche Fehlfunktion nicht zulasten des Entwicklers geht und du ihn von möglichen Schadenersatzforderungen freistellst.

*/

// Funktion zur Bereinigung der PHP-Version auf Haupt- und Minor-Version
function getMajorMinorVersion($version) {
    $parts = explode('.', $version);
    return isset($parts[0], $parts[1]) ? "{$parts[0]}.{$parts[1]}" : $version;
}

// Funktion zur Suche nach der PHP-CLI-Version, die mit der Server-Version übereinstimmt
function findMatchingPhpCliPath($phpVersionWebMajorMinor) {
    $commonPaths = [
        '/usr/bin/php',
        '/usr/local/bin/php',
        '/bin/php',
        '/opt/php/bin/php',
        '/usr/php/bin/php',
        '/opt/local/bin/php'
    ];

    // Zusätzliche Varianten für Provider mit spezifischen PHP-CLI-Namen
    $majorVersion = str_replace('.', '', $phpVersionWebMajorMinor); // z. B. "8.1" -> "81"
    $commonPaths = array_merge($commonPaths, [
        "/usr/bin/php{$phpVersionWebMajorMinor}", // Format: php8.1
        "/usr/local/bin/php{$phpVersionWebMajorMinor}",
        "/bin/php{$phpVersionWebMajorMinor}",
        "/opt/php/bin/php{$phpVersionWebMajorMinor}",
        "/usr/php/bin/php{$phpVersionWebMajorMinor}",
        "/opt/local/bin/php{$phpVersionWebMajorMinor}",
        "/usr/bin/php{$majorVersion}", // Format: php81
        "/usr/local/bin/php{$majorVersion}",
        "/bin/php{$majorVersion}",
        "/opt/php/bin/php{$majorVersion}",
        "/usr/php/bin/php{$majorVersion}",
        "/opt/local/bin/php{$majorVersion}"
    ]);

    foreach ($commonPaths as $path) {
        if (file_exists($path) && is_executable($path)) {
            // Prüfe die Version der CLI
            $output = [];
            $returnVar = 0;
            exec("$path -v", $output, $returnVar);

            if ($returnVar === 0 && isset($output[0])) {
                if (preg_match('/PHP\s+([\d\.]+)/', $output[0], $matches)) {
                    $cliVersion = getMajorMinorVersion($matches[1]);
                    if ($cliVersion === $phpVersionWebMajorMinor) {
                        return $path; // Pfad gefunden, der mit der Webserver-Version übereinstimmt
                    }
                }
            }
        }
    }
    return null; // Keine Übereinstimmung gefunden
}

// Funktion zur Überwachung und Extraktion der Matomo-Version aus der Ausgabe
function getMatomoVersion($phpCliPath, $consolePath) {
    $output = [];
    $returnVar = 0;

    // Führe die "console"-Datei aus
    exec("$phpCliPath $consolePath", $output, $returnVar);

    if ($returnVar === 0 && isset($output[0])) {
        // Prüfe die erste Zeile der Ausgabe auf das Muster "Matomo X.Y.Z"
        if (preg_match('/^Matomo\s+(\d+\.\d+\.\d+)/', $output[0], $matches)) {
            return $matches[1]; // Matomo-Version gefunden
        }
    }
    return null;
}

// Informationen sammeln
$currentPath = __DIR__;
$phpVersionWeb = phpversion();
$phpVersionWebMajorMinor = getMajorMinorVersion($phpVersionWeb);
$phpCliPath = findMatchingPhpCliPath($phpVersionWebMajorMinor);
$consolePath = $currentPath . DIRECTORY_SEPARATOR . 'console';
$matomoVersion = null;
$cronJobCommand = null;

// Prüfe, ob die Datei "console" existiert und ausführbar ist
if ($phpCliPath && file_exists($consolePath) && is_file($consolePath) && is_readable($consolePath)) {
    $matomoVersion = getMatomoVersion($phpCliPath, $consolePath);

    // Wenn eine Matomo-Version gefunden wurde, generiere den Cronjob-Befehl
    if ($matomoVersion) {
        $hostname = $_SERVER['HTTP_HOST'];
        $cronJobCommand = "$phpCliPath $consolePath core:archive --matomo-domain=$hostname";
    }
}

// HTML-Ausgabe
?>
<!DOCTYPE html>
<html lang="de">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Server-Informationen</title>
    <style>body{font-family:Arial,sans-serif;background-color:#f4f4f9;color:#333;margin:0;padding:20px;}.container{display:table;margin:0 auto;min-width:600px;margin:0 auto;background:#fff;padding:20px;border-radius:8px;box-shadow:0 2px 8px rgba(0,0,0,0.1);}h1{text-align:center;color:#004380;}.info{margin:10px 0;padding:10px;background:#f9f9f9;border-left:4px solid #004380;border-radius:4px;}.info strong{color:#333;}.alert{color:#dd0000;}
    </style>
</head>
<body>
    <div class="container">
    <h1>Server-Informationen</h1>
    <div class="info"><strong>Aktueller Pfad:</strong> <?php echo htmlspecialchars($currentPath); ?></div>
	<div class="info"><strong>PHP-Version (Webserver):</strong> <?php echo htmlspecialchars($phpVersionWebMajorMinor); ?></div>
	<div class="info"><strong>Pfad zur PHP-CLI (identisch mit Webserver):</strong> <?php echo $phpCliPath ? htmlspecialchars($phpCliPath) : '<span class="alert">Keine passende PHP-CLI-Version gefunden.</span>'; ?></div>
	<div class="info"><strong>Matomo-Version:</strong> <?php echo $matomoVersion ? htmlspecialchars($matomoVersion) : '<span class="alert">Keine Matomo-Version gefunden.</span>'; ?></div>
	<div class="info"><strong>vermutlicher Cronjob-Befehl:</strong><br><br><?php echo $cronJobCommand ? '<pre>'.htmlspecialchars($cronJobCommand).'</pre>' : '<span class="alert">Konnte nicht ermittelt werden.</span>'; ?></div>
    <p><small>(w) 2025 Joachim Nickel – <a href="https://www.chop.de/?ccam=cronjob-test&cmed=link&csrc=website">chop solutions GmbH</a><br>Lizenziert unter GNU GPL3 https://www.gnu.org/licenses/gpl-3.0.html</small></p>
    </div>
</body>
</html>
